﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.Text;
using System.Windows.Forms;
using System.Drawing;
using System.IO;

using MapApp.Model;

namespace MapApp.Ui.Controls
{
    public class MapClickEventArgs : EventArgs
    {
        public MapLocation MapLocation;
        public Point MousePosition;
        public MouseButtons Button;
    }

    public partial class GoogleMapControl : WebBrowser
    {
        private Map _map = new Map();

        public Map Map
        {
            get { return _map; }
            set
            {
                if (_map != value)
                {
                    _map = value;
                    if (_map != null)
                    {
                        _map.MapLocations.RaiseListChangedEvents = true;
                        _map.MapLocations.ListChanged += new ListChangedEventHandler(MapLocations_ListChanged);
                        DrawMapLocationCollection(_map.MapLocations);
                    }
                }
            }
        }

        void MapLocations_ListChanged(object sender, ListChangedEventArgs e)
        {
            DrawMapLocationCollection((MapLocationCollection)sender);
        }

        private void DrawMapLocationCollection(MapLocationCollection locations)
        {
            if (locations == null) return;
            foreach (MapLocation loc in locations)
            {
                if (loc.Id == 0)
                {
                    loc.Id = NewMarker(loc);
                }
            }

        }

        private MapLocation _initialCenterAndZoom =
                new MapLocation("", -31.94, 115.83, 3);

        private static string INIT_CENTER_AND_ZOOM_TOKEN = "//<!--SetInitialCenterAndZoom-->";
        private static string INIT_MARKERS_TOKEN = "//<!--InitialMarkers-->";

        private static string mapFilePath = AppUtil.tempDir + "/map.html";
        private static string pdmarkerFilePath = AppUtil.tempDir + "/pdmarker.js";
        private static string cssFilePath = AppUtil.tempDir + "/style.css";

        public delegate void OnMapMouseMove(GoogleMapControl sender, MapLocation loc);
        public event OnMapMouseMove MapMouseMove;

        public delegate void OnMapClick(GoogleMapControl sender, MapClickEventArgs e);
        public event OnMapClick MapClick;

        public GoogleMapControl()
        {
            InitializeComponentAndCustomInitialize();
        }

        public GoogleMapControl(IContainer container)
        {
            container.Add(this);

            InitializeComponentAndCustomInitialize();
        }

        /// <summary>
        /// Used in constructors
        /// </summary>
        private void InitializeComponentAndCustomInitialize()
        {
            InitializeComponent();
            CustomInitialize();
        }

        private void CustomInitialize()
        {
            this.ScrollBarsEnabled = false;

            // Copy the stylesheet to temp directory:
            AppUtil.writeResourceToDisk("MapApp.Ui.Controls.Resources.html.style.css", cssFilePath);
            //// Copy the map.html & pdmarker.js files from resources to temp dir:


            AppUtil.writeResourceToDisk("MapApp.Ui.Controls.Resources.html.pdmarker.js", pdmarkerFilePath);

            if (_map != null)
            {
                _map.MapLocations.RaiseListChangedEvents = true;
                _map.MapLocations.ListChanged += new ListChangedEventHandler(MapLocations_ListChanged);
                _map.MapLocationsChanged += new Map.MapLocationsChangedEventHandler(_map_MapLocationsChanged);
                DrawMapLocationCollection(_map.MapLocations);
            }


        }

        private string InitialMarkersJavaScript()
        {
            string result = "";
            foreach (MapLocation loc in Map.MapLocations)
            {
                result += String.Format("newMarker({0}, {1}, \"{2}\");\n",
                    loc.Lat, loc.Lng, loc.Label == null ? "" : loc.Label);
            }
            return result;
       }

        void _map_MapLocationsChanged(object sender, EventArgs e)
        {
            DrawMapLocationCollection(sender as MapLocationCollection);
        }

        public MapLocation InitialCenterAndZoom
        {
            set
            {
                _initialCenterAndZoom = value;
            }
        }


        /// <summary>
        /// This *must* be called after all the InitialCenterAndZoom, and MapLocations 
        /// have been set, to display the map.
        /// </summary>
        public void DisplayMap()
        {
            // _map.setCenter(new GLatLng(-31.94, 115.83), 3);
            string sSetCenter =
                "_map.setCenter(new GLatLng(" + _initialCenterAndZoom.Lat +
                ", " + _initialCenterAndZoom.Lng +
                "), " + _initialCenterAndZoom.ZoomLevel +
                ");";
            Dictionary<string, string> replacements = new Dictionary<string, string>();
            replacements.Add(INIT_CENTER_AND_ZOOM_TOKEN, sSetCenter);
            replacements.Add(INIT_MARKERS_TOKEN, InitialMarkersJavaScript());
            AppUtil.writeResourceToDisk("MapApp.Ui.Controls.Resources.html.map.html",
                mapFilePath, replacements);

            Url = new Uri(mapFilePath);
        }


        //void Document_MouseDown(object sender, HtmlElementEventArgs e)
        //{
        //    MessageBox.Show("down" + ", x: " + e.MousePosition.X + ", Y: " + e.MousePosition.Y);
        //}


        public void ZoomTo(MapLocation loc)
        {
            Object[] objArray = { loc.Lat, loc.Lng, loc.ZoomLevel };
            Document.InvokeScript("zoomTo", objArray);
        }

        public void SetOverviewMapVisible(bool visible)
        {
            Object[] objArray = { visible };
            Document.InvokeScript("setOverviewMapVisible", objArray);
        }

        public void SetScaleBarVisible(bool visible)
        {
            Object[] objArray = { visible };
            Document.InvokeScript("setScaleBarVisible", objArray);
        }

        public int NewMarker(MapLocation loc)
        {
            string toolTip = string.IsNullOrEmpty(loc.Label) ? "" : loc.Label;
            Object[] objArray = { loc.Lat, loc.Lng, toolTip };
            int id = Convert.ToInt32(Document.InvokeScript("newMarker", objArray));
            return id;
        }

        public static MapLocation CenterAndZoomFromString(string str)
        {
            MapLocation result = new MapLocation("", 0.0, 0.0, 0);
            try
            {
                string[] sArray = str.Split(':');
                result.Lat = Convert.ToDouble(sArray[0]);
                result.Lng = Convert.ToDouble(sArray[1]);
                result.ZoomLevel = Convert.ToInt32(sArray[2]);
            }
            catch { } // ignore exceptions
            return result;
        }

        public static string CenterAndZoomToString(MapLocation loc)
        {
            string result = "";
            if (loc == null) return result;
            result = loc.Lat + ":" + loc.Lng + ":" + loc.ZoomLevel;
            return result;
        }

        public MapLocation GetCenterAndZoom()
        {
            string s = (string)Document.InvokeScript("getCenterAndZoom");
            double lat = 0.0, lng = 0.0;
            int zoom = 0;
            try
            {
                if (s != null)
                {
                    string[] sArray = s.Split(':');
                    lat = Convert.ToDouble(sArray[0]);
                    lng = Convert.ToDouble(sArray[1]);
                    zoom = Convert.ToInt32(sArray[2]);
                }
            }
            catch { } // ignore exceptions
            MapLocation result = new MapLocation("", lat, lng, zoom);
            return result;
        }

        public void SetCenterAndZoom(MapLocation loc)
        {
            Object[] objArray = { loc.Lat, loc.Lng, loc.ZoomLevel };
            Document.InvokeScript("setCenterAndZoom", objArray);
        }

        private MapLocation PixelToMapLocation(int pixelX, int pixelY)
        {
            Object[] objArray = { pixelX, pixelY };
            string llString = (string)Document.InvokeScript("pixelToLatLng", objArray);
            double lat = 0.0, lng = 0.0;
            if (llString != null)
            {
                string[] llArray = llString.Split(':');
                try
                {
                    lat = Convert.ToDouble(llArray[0]);
                    lng = Convert.ToDouble(llArray[1]);
                }
                catch { } // ignore exceptions
            }
            MapLocation result = new MapLocation("", lat, lng);
            return result;
        }


        private void GoogleMapControl_DocumentCompleted(object sender, WebBrowserDocumentCompletedEventArgs e)
        {
            //Document.MouseDown += new HtmlElementEventHandler(Document_MouseDown);
            Document.MouseMove += new HtmlElementEventHandler(Document_MouseMove);
            Document.Click += new HtmlElementEventHandler(Document_Click);
            Document.MouseUp += new HtmlElementEventHandler(Document_MouseUp);

            //Document.AttachEventHandler("mousemove", new EventHandler(Document_MouseMove));
        }

        void Document_MouseMove(object sender, HtmlElementEventArgs e)
        {
            if (MapMouseMove != null)
            {
                MapLocation loc = PixelToMapLocation(e.MousePosition.X, e.MousePosition.Y);
                MapMouseMove(this, loc);
            }
        }

        void Document_MouseUp(object sender, HtmlElementEventArgs e)
        {
            // Generated by javascript on right-mouse click.
            // Generate a MapClick right-mouse click event:
            if (MapClick != null)
            {
                if (e.MouseButtonsPressed == MouseButtons.Right)
                {
                    MapClickEventArgs args = new MapClickEventArgs();
                    args.MousePosition = e.MousePosition;
                    args.MapLocation = PixelToMapLocation(e.MousePosition.X, e.MousePosition.Y);
                    args.Button = MouseButtons.Right;
                    MapClick(this, args);
                }
            }
        }

        void Document_Click(object sender, HtmlElementEventArgs e)
        {
            // Generated by javascript on left-mouse click.
            // Generate a MapClick left-mouse click event:
            if (MapClick != null)
            {
                MapClickEventArgs args = new MapClickEventArgs();
                args.MapLocation = PixelToMapLocation(e.MousePosition.X, e.MousePosition.Y);
                args.Button = MouseButtons.Left;
                MapClick(this, args);
            }
        }

    }
}
